@@ -35,13 +35,12 @@ int vbuf__append_quoted1035(vbuf *vb, const byte *buf, int len) {
     qbuf[0]= 0;
     for (i=0; i<len; i++) {
       ch= buf[i];
-      if (ch == '.' || ch == '"' || ch == '(' || ch == ')' ||
-	  ch == '@' || ch == ';' || ch == '$' || ch == '\\') {
-	sprintf(qbuf,"\\%c",ch);
-	break;
-      } else if (ch <= ' ' || ch >= 127) {
+      if (ch <= ' ' || ch >= 127) {
 	sprintf(qbuf,"\\%03o",ch);
 	break;
+      } else if (!ctype_domainunquoted(ch)) {
+	sprintf(qbuf,"\\%c",ch);
+	break;
       }
     }
     if (!adns__vbuf_append(vb,buf,i) || !adns__vbuf_append(vb,qbuf,strlen(qbuf)))
